/*
 * MethodWrapper.java
 *  
 * Copyright 2007 C.A. Meijer
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.hammingweight.hammockmaker;

import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.Arrays;

/**
 * 
 * @author Carl
 */
public class MethodWrapper extends AMethodWrapper {

	private String methodId;

	private Method method;

	private static String getUnqualifiedName(Class<?> clazz) {
		String prepend = "";
		while (clazz.isArray()) {
			prepend += "ARRAY_";
			clazz = clazz.getComponentType();
		}

		String s = clazz.toString();
		int index = s.lastIndexOf('.');
		if (index >= 0) {
			s = s.substring(index + 1);
		}

		return prepend + s;
	}

	/** Creates a new instance of MethodWrapper */
	public MethodWrapper(String methodId, Method method) {
		super(new Arguments(method.getParameterTypes()), new Exceptions(method
				.getExceptionTypes()));
		this.methodId = methodId;
		this.method = method;
	}

	public String getReturnType() {
		return ClassWrapper.classToString(method.getReturnType());
	}

	public String getObjectReturnType() {
		if (this.method.getReturnType().isArray()) {
			return ClassWrapper.classToString(this.method.getReturnType())
					+ ".class";
		}
		if (!this.method.getReturnType().isPrimitive()) {
			return ClassWrapper.classToString(method.getReturnType()) + ".class";
		}

		Class<?> clazz = this.method.getReturnType();
		String type = ClassWrapper.getWrapperForPrimitiveType(clazz);
		return type + ".class";
	}

	public String getDeclaration() {
		String s = "public " + this.getReturnType() + " ";
		s += this.method.getName() + "(";
		s += this.getArguments().getArgList();
		s += ") ";
		s += this.getExceptions().getThrowsClause();

		return s;
	}

	public String getName() {
		return this.method.getName();
	}

	public String getMethodIdentifier() {
		return this.methodId;
	}

	public static String getMethodIdentifier(Method method) {
		String s = "MTHD_" + method.getName();
		Class<?>[] argTypes = method.getParameterTypes();
		if (argTypes.length > 0) {
			s += "_$";
		}
		for (Class<?> clazz : argTypes) {
			s += "_" + getUnqualifiedName(clazz).toUpperCase();
		}
		
		String t = s.substring(0, 5);
		char d = s.charAt(4);
		for (int i = 5; i < s.length(); i++) {
			char c = s.charAt(i);
			if (c == '[') {
				t += "ARRAY_";
				continue;
			}
			if (!Character.isLetter(c) && !Character.isDigit(c) && c != '$') {
				t += '_';
				continue;
			}
			if (Character.isUpperCase(c) && Character.isLowerCase(d)) {
				t += "_";
			}
			t += Character.toUpperCase(c);
			d = c;
		}

		return t;
	}

	public boolean isAbstract() {
		return Modifier.isAbstract(method.getModifiers());
	}

	public boolean isReturnTypeVoid() {
		return (method.getReturnType() == void.class);
	}

	public boolean isReturnTypePrimitive() {
		return (method.getReturnType().isPrimitive());
	}

   public String getReturnClass() {
		Class<?> clazz = method.getReturnType();
		if (clazz == boolean.class) {
			return "BOOLEAN_CLASS";
		} else if (clazz == byte.class) {
			return "BYTE_CLASS";
		} else if (clazz == char.class) {
			return "CHARACTER_CLASS";
		} else if (clazz == short.class) {
			return "SHORT_CLASS";
		} else if (clazz == int.class) {
			return "INTEGER_CLASS";
		} else if (clazz == long.class) {
			return "LONG_CLASS";
		} else {
			return "OBJECT_CLASS";
		}
   }
   
	public String getPrimitiveReturnTypeCast() {
		Class<?> clazz = method.getReturnType();
		if (clazz == boolean.class) {
			return "((Boolean)retVal).booleanValue()";
		} else if (clazz == byte.class) {
			return "((Byte)retVal).byteValue()";
		} else if (clazz == char.class) {
			return "((Character)retVal).charValue()";
		} else if (clazz == short.class) {
			return "((Short)retVal).shortValue()";
		} else if (clazz == int.class) {
			return "((Integer)retVal).intValue()";
		} else if (clazz == long.class) {
			return "((Long)retVal).longValue()";
		} else if (clazz == float.class) {
			return "((Float)retVal).floatValue()";
		} else if (clazz == double.class) {
			return "((Double)retVal).doubleValue()";
		} else {
			return null;
		}
	}

	public int hashCode() {
		return this.methodId.hashCode();
	}

	public boolean equals(Object o) {
		if (!(o instanceof MethodWrapper)) {
			return false;
		}

		MethodWrapper mw = (MethodWrapper) o;
		if (!this.method.getName().equals(mw.method.getName())) {
			return false;
		}

		Class<?>[] params1 = this.method.getParameterTypes();
		Class<?>[] params2 = mw.method.getParameterTypes();

		return Arrays.equals(params1, params2);
	}
	
	public boolean isFinal() {
		return Modifier.isFinal(this.method.getModifiers());
	}
	
	public void setMethodId(String methodId) {
		this.methodId = methodId;
	}
}
